# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-17 07:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sampleapp', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='foo',
            name='yesorno',
            field=models.BooleanField(default=True, verbose_name='Check for yes'),
        ),
    ]



# coding: utf-8
from __future__ import print_function, division, absolute_import, unicode_literals

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .helpers import get_object_repr


ADDITION = 1
CHANGE = 2
DELETION = 3

ACTION_TYPE_CHOICES = ((ADDITION, _("Addition")), (CHANGE, _("Change")), (DELETION, _("Deletion")))

ACTION_MESSAGES = {
    ADDITION: _("""{} has added '{}'"""),
    CHANGE: _("""{} has updated '{}'"""),
    DELETION: _("""{} has deleted '{}'"""),
}


@python_2_unicode_compatible
class Entry(models.Model):
    """Entry log that will save an action made by a user (form submission, or any other custom action)

    Query examples:
        You can extend your model with HistoryBaseModel defined below
        or directly query the Entry model.

        >>> entries = Entry.get_for_model(my_instance)  # get all entries for an instance
        >>> entries = entries.filter(created_by=my_user)  # filter entries on a specific user

    """

    created_date = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(settings.AUTH_USER_MODEL, related_name="history_entries", null=True, blank=True)

    object_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey("object_type", "object_id")
    object_repr = models.CharField(max_length=200)

    action_type = models.PositiveSmallIntegerField(choices=ACTION_TYPE_CHOICES)
    short_message = models.TextField(null=True, blank=True)

    class Meta:
        ordering = ("-created_date",)

    def __str__(self):
        return self.short_message

    @staticmethod
    def create(user, content_object, action_type=CHANGE, changelog=None, short_message=None):
        """Saves an Entry log history of the 'content_object'

        Args:
            user (auth.User): The user who made the action
            content_object (models.Model): The related model object
            action_type (int): Action type (see ACTION_TYPE_CHOICES)
            changelog (list, optional): List of ChangedData attributes that will be created
            object_repr (str, optional): A representation of the content_object
            short_message (str, optional): A short message that can be displayed if no changelog is available

        Returns:
            Entry: The freshly created Entry
        """
        changelog = changelog or []
        entry = Entry.objects.create(
            created_by=user,
            action_type=action_type,
            object_type=ContentType.objects.get_for_model(content_object),
            object_id=content_object.id,
            object_repr=get_object_repr(content_object),
        )

        if not short_message:
            short_message = ACTION_MESSAGES[action_type].format(entry.get_user_full_name(), entry.object_repr)

        entry.short_message = short_message
        entry.save()
        [ChangedData(entry=entry, **data).save() for data in changelog]
        return entry

    @staticmethod
    def get_for_model(obj):
        return Entry.objects.filter(object_id=obj.pk, object_type=ContentType.objects.get_for_model(obj))

    def get_user_full_name(self):
        """Returns the human readable name of the user"""
        if self.created_by:
            return self.created_by.get_full_name() or self.created_by.username
        return _("Anonymous")

    def is_addition(self):
        return self.action_type == ADDITION

    def is_change(self):
        return self.action_type == CHANGE

    def is_deletion(self):
        return self.action_type == DELETION


@python_2_unicode_compatible
class ChangedData(models.Model):
    """Store the changed datas of a forms.ModelForm
    The label, initial value and changed value intend to be human readable,
    so they can be used directly to show to end-users.
    """

    entry = models.ForeignKey(Entry)
    label = models.CharField(_("Label"), max_length=200)
    initial_value = models.TextField(_("Initial value"), blank=True, null=True)
    changed_value = models.TextField(_("Updated value"), blank=True, null=True)

    def __str__(self):
        return self.label


class HistoryBaseModel(models.Model):
    """Base model that implements utilities to query history entries of your models.
    Just extends your model with this one :

        >> class MyModel(HistoryBaseModel):
        >>     pass
        >>
        >> inst = MyModel.objects.get(...)
        >> inst.get_history_entries()  # Returns <Entry queryset>

    """

    def get_history_entries(self):
        """Returns all the history Entries for that model"""
        return Entry.get_for_model(self)

    def log_custom_history(self, user, message):
        """Save a message in history for that instance"""
        return Entry.create(user=user, content_object=self, short_message=message)

    class Meta:
        abstract = True



# coding: utf-8
from __future__ import print_function, division, absolute_import, unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from modelformhistory.models import HistoryBaseModel

FOO_CHOICES = (("ok", "It's OK"), ("nok", "It's not OK"))


@python_2_unicode_compatible
class Bar(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Baz(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Foo(HistoryBaseModel, models.Model):
    name = models.CharField("Name of this instance", max_length=100)
    integer = models.IntegerField("Type your favorite integer")
    choose_somthing = models.CharField("Make your choice", max_length=100, choices=FOO_CHOICES, default="ok")
    bar = models.ForeignKey(Bar, verbose_name="Name of the bar", null=True, blank=True)
    baz = models.ManyToManyField(Baz, verbose_name="Select some baz", blank=True)
    yesorno = models.BooleanField("Check for yes", default=True)
    picture = models.ImageField(null=True, blank=True)

    def __str__(self):
        return self.name



# coding: utf-8
from __future__ import print_function, division, absolute_import, unicode_literals

from django import forms
from django.db import models
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _


def get_object_repr(instance):
    return "%s %s" % (type(instance)._meta.verbose_name.title(), instance)


def get_human_value_for_field(bounded_field, value):
    if value is None:
        return _("Empty")

    if isinstance(bounded_field.field, forms.FileField):
        return getattr(value, "name", "{}".format(value))

    value = bounded_field.field.clean(value)

    if not isinstance(value, str) and hasattr(value, "__iter__"):
        # iterable
        return ", ".join([force_text(i) for i in value])

    elif isinstance(value, models.Model):
        # FK or single M2M
        return force_text(value)

    elif isinstance(bounded_field.field, (forms.MultipleChoiceField, forms.ChoiceField)):
        # choiceField
        return dict(bounded_field.field.choices).get(value)

    elif type(value) == bool:
        # Boolean
        return {True: _("Yes"), False: _("No")}[value]

    return value



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-13 08:08
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ChangedData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=200, verbose_name='Label')),
                ('initial_value', models.TextField(blank=True, null=True, verbose_name='Initial value')),
                ('changed_value', models.TextField(blank=True, null=True, verbose_name='Updated value')),
            ],
        ),
        migrations.CreateModel(
            name='Entry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('object_repr', models.CharField(max_length=200)),
                ('action_type', models.PositiveSmallIntegerField(choices=[(1, 'Addition'), (2, 'Change'), (3, 'Deletion')])),
                ('short_message', models.TextField(blank=True, null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='history_entries', to=settings.AUTH_USER_MODEL)),
                ('object_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ('-created_date',),
            },
        ),
        migrations.AddField(
            model_name='changeddata',
            name='entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='modelformhistory.Entry'),
        ),
    ]



from django.contrib import admin

from .forms import HistoryModelFormMixin
from .models import Entry, ChangedData


class HistoryModelAdmin(admin.ModelAdmin):
    """
    Base class ModelAdmin to automatically store any change in history.
    Extend your ModelAdmin class to start using it :

        class FooAdmin(HistoryModelAdmin):
            pass

    """

    def get_form(self, request, obj=None, **kwargs):
        form = super(HistoryModelAdmin, self).get_form(request, obj=None, **kwargs)

        class HistoryAdminForm(HistoryModelFormMixin, form):
            class Meta(form.Meta):
                pass

            def get_history_user(self):
                return request.user

        return HistoryAdminForm


class ChangedDataInline(admin.TabularInline):
    model = ChangedData
    extra = 0


class EntryAdmin(admin.ModelAdmin):
    search_fields = ("created_by__username", "short_message")
    list_display = ("created_date", "created_by", "action_type", "short_message")
    raw_id_fields = ("created_by",)
    inlines = [ChangedDataInline]


admin.site.register(Entry, EntryAdmin)



from django import forms

from modelformhistory.forms import HistoryModelFormMixin
from .models import Foo


class FooModelFormRequest(HistoryModelFormMixin, forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(FooModelFormRequest, self).__init__(*args, **kwargs)

    class Meta:
        model = Foo
        fields = "__all__"


class FooModelForm(HistoryModelFormMixin, forms.ModelForm):
    def __init__(self, user, *args, **kwargs):
        self.user = user
        super(FooModelForm, self).__init__(*args, **kwargs)

    def get_history_user(self):
        return self.user

    class Meta:
        model = Foo
        fields = "__all__"



# coding: utf-8
from __future__ import print_function, division, absolute_import, unicode_literals

# from .exceptions import SouvenirFormException
from .models import Entry, ADDITION, CHANGE
from .helpers import get_human_value_for_field


class HistoryModelFormMixin(object):
    """

    Extend your django modelform with that mixin.
    Place it in the first position.

        class FooForm(HistoryModelFormMixin, forms.ModelForm):
           ...

    If you pass a 'request' parameter o your form, the request.user will be
    saved in the history Entry. You can override this by implementing a 'get_history_user' method in your modelform.


    """

    class Meta:
        history_exclude = ()  # List of form field that won't be saved in the history

    def __init__(self, *args, **kwargs):
        super(HistoryModelFormMixin, self).__init__(*args, **kwargs)
        self._history_exclude = getattr(self.Meta, "history_exclude", ())
        self._history_user = self.get_history_user()
        self._history_action_type = self.get_history_action_type()

    def get_history_user(self):
        """
        Returns the auth.User that will be saved in the history entry.
        By default, searches in self.request.user, or return None.
        """
        if hasattr(self, "request"):
            return getattr(self.request, "user", None)

    def get_history_action_type(self):
        """Is it ADDITION or CHANGE ? Returns a int"""
        if self.instance.pk is None:
            return ADDITION
        else:
            return CHANGE

    def save(self, *args, **kwargs):
        obj = super(HistoryModelFormMixin, self).save(*args, **kwargs)
        if self.has_changed() or self.files:
            changelog = []
            if self._history_action_type == CHANGE:
                bounded_fields = dict([(f.name, f) for f in self])
                for fieldname in self.changed_data:
                    field = bounded_fields.get(fieldname)
                    initial_value = get_human_value_for_field(field, field.initial)
                    changed_value = get_human_value_for_field(field, field.data)
                    if initial_value != changed_value:
                        changelog.append(
                            {"label": field.label, "initial_value": initial_value, "changed_value": changed_value}
                        )

            Entry.create(
                user=self._history_user,
                content_object=self.instance,
                action_type=self._history_action_type,
                changelog=changelog,
            )
        return obj



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-06-14 04:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Bar",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name="Baz",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name="Foo",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=100, verbose_name="Name of this instance")),
                ("integer", models.IntegerField(verbose_name="Type your favorite integer")),
                (
                    "choose_somthing",
                    models.CharField(
                        choices=[("ok", "It's OK"), ("nok", "It's not OK")],
                        default="ok",
                        max_length=100,
                        verbose_name="Make your choice",
                    ),
                ),
                (
                    "bar",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sampleapp.Bar",
                        verbose_name="Name of the bar",
                    ),
                ),
                ("baz", models.ManyToManyField(blank=True, to="sampleapp.Baz", verbose_name="Select some baz")),
            ],
        ),
    ]



from setuptools import setup, find_packages

install_requires = []
description = ""

for file_ in ("README", "CHANGELOG"):
    with open("%s.md" % file_) as f:
        description += f.read() + "\n\n"


classifiers = [
    "Environment :: Web Environment",
    "Framework :: Django",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: BSD License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Software Development :: Libraries :: Python Modules",
]


setup(
    name="django-modelformhistory",
    version="1.0.5",
    url="https://github.com/EngageSports/django-modelformhistory",
    packages=find_packages(exclude=["tests"]),
    long_description_content_type="text/markdown",
    long_description=description,
    author="Martyn CLEMENT",
    author_email="martyn.clement@gmail.com",
    include_package_data=True,
    zip_safe=False,
    classifiers=classifiers,
    install_requires=install_requires,
)
